local MAJOR = "StarLibPluginColor-1.0"
local MINOR = 13

local PluginColor = LibStub:NewLibrary(MAJOR, MINOR)
if not PluginColor then return end

local Colorize, AggroColor, ThreatStatusColor, HPColor, PowerColor, ReputationColor, HostileColor, ClassColor, DifficultyColor

-- @name PluginColor:New
-- @usage PluginColor:New(environment)
-- @parma environment This will be the environment when loadstring is called.
-- @return Nothing
function PluginColor:New(environment)
	environment.Colorize = Colorize
	environment.ThreatStatusColor = ThreatStatusColor
	environment.AggroColor = AggroColor
	environment.HPColor = HPColor
	environment.PowerColor = PowerColor
	environment.ReputationColor = ReputationColor
	environment.HostileColor = HostileColor
	environment.ClassColor = ClassColor
	environment.DifficultyColor = DifficultyColor
	environment.gradient = PluginColor:RedThroughBlue()
end

function PluginColor:RedThroughBlue()
	local colors = {}

	-- Start with solid red:
	local r = 255;
	local g = 0;
	local b = 0;
	
	-- From red to yellow:
	local i = 0
	while i <= 255 do
		g = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i + 1
	end

	-- From yellow to green:
	i = 255
	while i >= 0 do
		r = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i - 1
	end
	
	-- From green to blue:
	i = 0
	while i <= 255 do
		b = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		g = g - 1
		i = i + 1
	end

	local subColors = {}
	
	for i = #colors, 1, -1 do
		if mod(i, 8) == 0 then
			tinsert(subColors, colors[i])
		end
	end
	
	for i = #subColors + 1, 100 do
		tinsert(subColors, colors[1])
	end

	return subColors
end

function Colorize(str, r, g, b)
	if not r then r = 1 end
	if not g then g = 1 end
	if not b then b = 1 end
	return ("|cff%02x%02x%02x%s|r"):format(r * 255, g * 255, b * 255, str)
end

function AggroColor(unit)
	local r, g, b = UnitSelectionColor(unit)
	return r, g, b
end

function ThreatStatusColor(status)
	local r, g, b = GetThreatStatusColor(status)
	return r, g, b
end

function HPColor(cur, max)
	local perc = cur / max
	local r1, g1, b1
	local r2, g2, b2
	if perc <= 0.5 then
		perc = perc * 2
		r1, g1, b1 = 1, 0, 0 
		r2, g2, b2 = 1, 1, 0
	else
		perc = perc * 2 - 1
		r1, g1, b1 = 1, 1, 0
		r2, g2, b2 = 0, 1, 0
	end
	local r, g, b = r1 + (r2 - r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
	if r < 0 then
		r = 0
	elseif r > 1 then
		r = 1
	end
	if g < 0 then
		g = 0
	elseif g > 1 then
		g = 1
	end
	if b < 0 then
		b = 0
	elseif b > 1 then
		b = 1
	end
	return r, g, b
end

local PowerBarColor = _G.PowerBarColor

function PowerColor(power_type)
	local color = PowerBarColor[power_type]
	local r,g,b
	if color then
		r, g, b = color.r,color.g,color.b
	else
		r, g, b = 0.7, 0.7, 0.7
	end
	return r, g, b
end

function ReputationColor(reaction)
  local color = FACTION_BAR_COLORS[reaction]
	if color then
		return color.r, color.g, color.b
	end
end

local FACTION_BAR_COLORS = _G.FACTION_BAR_COLORS
local HOSTILE_REACTION = 2
local NEUTRAL_REACTION = 4
local FRIENDLY_REACTION = 5

function HostileColor(unit)
	local r, g, b
	if not unit then
		r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
	else
		if UnitIsPlayer(unit) or UnitPlayerControlled(unit) then
			if UnitCanAttack(unit, "player") then
				-- they can attack me
				if UnitCanAttack("player", unit) then
					-- and I can attack them
					local t = FACTION_BAR_COLORS[HOSTILE_REACTION]
					r, g, b = t.r, t.g, t.b
				else
					-- but I can't attack them
--					r, g, b = unpack(FACTION_BAR_COLORS.civilian)
					r, g, b = 0.8, 0.8, 0.8
				end
			elseif UnitCanAttack("player", unit) then
				-- they can't attack me, but I can attack them
				local t = (FACTION_BAR_COLORS[NEUTRAL_REACTION])
				r, g, b = t.r, t.g, t.b
			elseif UnitIsPVP(unit) then
				-- on my team
				local t = FACTION_BAR_COLORS[FRIENDLY_REACTION]
				r, g, b = t.r, t.g, t.b
			else
				-- either enemy or friend, no violance
				-- r, g, b = unpack(PitBull4.ReactionColors.civilian)
				r, g, b = 0.8, 0.8, 0.8
			end
		elseif (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) or UnitIsDead(unit) then
			r, g, b = 0.5, 0.5, 0.5 -- TODO: We really need this to be globally configurable.
		else
			local reaction = UnitReaction(unit, "player")
			if reaction then
				local t
				if reaction >= 5 then
					t = FACTION_BAR_COLORS[FRIENDLY_REACTION]
				elseif reaction == 4 then
					t = FACTION_BAR_COLORS[NEUTRAL_REACTION]
				else
					t = FACTION_BAR_COLORS[HOSTILE_REACTION]
				end
				r, g, b = t.r, t.g, t.b
			else
				r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
			end
		end
	end
	return r, g, b
end

local RAID_CLASS_COLORS = _G.RAID_CLASS_COLORS

function ClassColor(unit)
	local r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
	local _, class = UnitClass(unit)
	local t = RAID_CLASS_COLORS[class]
	if t then
		r, g, b = t.r, t.g, t.b
	end
	return r, g, b
end

function DifficultyColor(unit)
	local level
	level = UnitLevel(unit)
	if level <= 0 then
		level = 99
	end
	local color = GetQuestDifficultyColor(level)
	return color.r, color.g, color.b
end


